!{\src2tex{textfont=tt}}
!!****f* m_abi_linalg/abi_xpotrf
!! NAME
!!  abi_xpotrf
!!
!! FUNCTION
!!  abi_xpotrf is the generic function for computing the
!! Cholesky factorization of a real symmetric (or hermitian)
!!   positive definite matrix A.
!!   The factorization has the form
!!     A = U**T * U,  if UPLO = 'U', or
!!     A = L  * L**T,  if UPLO = 'L',
!!   where U is an upper triangular matrix and L is lower triangular.
!! In many cases it assumes that data are contiguous in memory
!!
!! COPYRIGHT
!!  Copyright (C) 2001-2013 ABINIT group (LNguyen,FDahm (CS))
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~ABINIT/Infos/copyright
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

!!***

!!****f* m_abi_linalg/abi_dpotrf
!! NAME
!! abi_dpotrf
!!
!! FUNCTION
!! 
!!
!! COPYRIGHT
!!   Copyright (C) 1998-2013 ABINIT group (LNguyen, FDahm (CS) )
!!   this file is distributed under the terms of the
!!   gnu general public license, see ~abinit/COPYING
!!   or http://www.gnu.org/copyleft/gpl.txt .
!!   for the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!   
!!
!! PARENTS
!!      
!!
!! CHILDREN
!!
!! SOURCE
!!
  subroutine abi_dpotrf(uplo,n,a,lda,info)
    use defs_basis
#if defined HAVE_LINALG_PLASMA
    !use plasma, plasma_dp => dp
    use iso_c_binding
#endif

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'abi_dpotrf'
!End of the abilint section

    implicit none
    !Arguments ------------------------------------
    character(len=1), intent(in) :: uplo
    integer, intent(in) :: n
    real(dp), intent(inout) :: a
    integer, intent(in) :: lda
    integer, intent(out) :: info
#if defined HAVE_LINALG_PLASMA
    !Optional Arguments ------------------------------------
    logical,external :: LSAME
    integer :: uplo_plasma_a
    include 'plasmaf.h'
#endif
    ! *********************************************************************
    !    write(*,*) "  abi_dpotrf => PLASMA dpotrf will be called "
#if defined HAVE_LINALG_PLASMA
    if(LSAME(uplo,'U')) then
       uplo_plasma_a = PlasmaUpper
    else
       uplo_plasma_a = PlasmaLower
    end if
    call PLASMA_dpotrf(uplo_plasma_a,n,a,lda,info)
#else
    call dpotrf(uplo,n,a,lda,info)
#endif
    return
  end subroutine abi_dpotrf
!!***

!!****f* m_abi_linalg/abi_zpotrf
!! NAME
!! abi_zpotrf
!!
!! FUNCTION
!! 
!!
!! COPYRIGHT
!!   Copyright (C) 1998-2013 ABINIT group (LNguyen, FDahm (CS) )
!!   this file is distributed under the terms of the
!!   gnu general public license, see ~abinit/COPYING
!!   or http://www.gnu.org/copyleft/gpl.txt .
!!   for the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!   
!!
!! PARENTS
!!      
!!
!! CHILDREN
!!
!! SOURCE
!!
  subroutine abi_zpotrf(uplo,n,a,lda,info)
    use defs_basis
#if defined HAVE_LINALG_PLASMA
    !use plasma, plasma_dp => dp
    use iso_c_binding
#endif

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'abi_zpotrf'
!End of the abilint section

    implicit none
    !Arguments ------------------------------------
    character(len=1), intent(in) :: uplo
    integer, intent(in) :: n
    complex(dpc), intent(inout) :: a
    integer, intent(in) :: lda
    integer, intent(out) :: info
#if defined HAVE_LINALG_PLASMA    
    !Optional Arguments ------------------------------------
    logical,external :: LSAME
    integer :: uplo_plasma_a
    include 'plasmaf.h'
#endif

    ! *********************************************************************
!    write(*,*) "  abi_zpotrf => PLASMA zpotrf will be called "
#if defined HAVE_LINALG_PLASMA    
    if(LSAME(uplo,'U')) then
       uplo_plasma_a = PlasmaUpper
    else
       uplo_plasma_a = PlasmaLower
    end if
    call PLASMA_zpotrf(uplo_plasma_a,n,a,lda,info)
#else
    call zpotrf(uplo,n,a,lda,info)
#endif
    return
  end subroutine abi_zpotrf
!!***

!!****f* m_abi_linalg/abi_zpotrf_2d
!! NAME
!! abi_zpotrf_2d
!!
!! FUNCTION
!! 
!!
!! COPYRIGHT
!!   Copyright (C) 1998-2013 ABINIT group (LNguyen, FDahm (CS) )
!!   this file is distributed under the terms of the
!!   gnu general public license, see ~abinit/COPYING
!!   or http://www.gnu.org/copyleft/gpl.txt .
!!   for the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!   
!!
!! PARENTS
!!      
!!
!! CHILDREN
!!
!! SOURCE
!!
  subroutine abi_zpotrf_2d(uplo,n,a,lda,info)
    use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'abi_zpotrf_2d'

!End of the abilint section

    implicit none

    !Arguments ------------------------------------
    character(len=1), intent(in) :: uplo
    integer, intent(in) :: n
    complex(dpc), intent(inout) :: a(:,:)
    integer, intent(in) :: lda
    integer, intent(out) :: info

    ! *********************************************************************
!    write(*,*) "  abi_zpotrf_2d  => abi_zpotrf will be called "
    call abi_zpotrf(uplo,n,a(1,1),lda,info)
    
    return
  end subroutine abi_zpotrf_2d
!!***

!!****f* m_abi_linalg/abi_d2zpotrf
!! NAME
!! abi_d2zpotrf
!!
!! FUNCTION
!! 
!!
!! COPYRIGHT
!!   Copyright (C) 1998-2013 ABINIT group (LNguyen, FDahm (CS) )
!!   this file is distributed under the terms of the
!!   gnu general public license, see ~abinit/COPYING
!!   or http://www.gnu.org/copyleft/gpl.txt .
!!   for the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!   
!!
!! PARENTS
!!      
!!
!! CHILDREN
!!
!! SOURCE
!!
  subroutine abi_d2zpotrf(uplo,n,a,lda,info,x_cplx)
    use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'abi_d2zpotrf'
!End of the abilint section

    implicit none

    !Arguments ------------------------------------
    character(len=1), intent(in) :: uplo
    integer, intent(in) :: n
    real(dp), intent(inout) :: a(:,:)
    integer, intent(in) :: lda
    integer, intent(out) :: info
    integer, intent(in), optional :: x_cplx
    !Local Variables -----------------------------
    integer  :: cplx_
#if defined HAVE_LINALG_PLASMA
    !Optional Arguments ------------------------------------
    logical,external :: LSAME
    integer :: uplo_plasma_a
    include 'plasmaf.h'
#endif

    ! *********************************************************************
    
    cplx_=1 ; if(PRESENT(x_cplx)) cplx_ = x_cplx
    
#if defined HAVE_LINALG_PLASMA
    if(LSAME(uplo,'U')) then
       uplo_plasma_a = PlasmaUpper
    else
       uplo_plasma_a = PlasmaLower
    end if
    if(cplx_ == 2) then
       call PLASMA_zpotrf(uplo_plasma_a,n,a,lda,info)
    else
       call PLASMA_dpotrf(uplo_plasma_a,n,a,lda,info)
    end if

#else
    if(cplx_ == 2) then
       call zpotrf(uplo,n,a,lda,info)
    else
       call dpotrf(uplo,n,a,lda,info)
    end if
#endif
    
    return
  end subroutine abi_d2zpotrf
!!***
