!{\src2tex{textfont=tt}}
!!****f* ABINIT/psp7wvl2
!! NAME
!! psp7wvl2
!!
!! FUNCTION
!! Compute data related to pspcod=7 or 17 ("PAW pseudopotentials") 
!! for wavelets
!!
!! COPYRIGHT
!! Copyright (C) 2013 ABINIT group (TRangel)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!  ipsp= index of pseudopotential
!!  wvl_crmult,wvl_frmult= variables definining the fine and coarse grids in a wavelets calculation
!!
!! OUTPUT
!! Stores HGH parameters to be used with wavelets
!!
!! SIDE EFFECTS
!!
!! NOTES
!!  
!!
!! PARENTS
!!      pspatm_abinit
!!
!! CHILDREN
!!      eleconf,psp_from_data,wrtout
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

subroutine psp7wvl2(ipsp,psps,wvl_crmult,wvl_frmult)

 use defs_basis
 use defs_datatypes
 use m_errors
 use m_splines
 use m_profiling

#if defined HAVE_DFT_LIBXC
  use libxc_functionals
#endif

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'psp7wvl2'
 use interfaces_14_hidewrite
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: ipsp
 real(dp),intent(in) :: wvl_crmult,wvl_frmult
 type(pseudopotential_type),intent(inout) :: psps
!arrays


!Local variables ------------------------------
!scalars
#if defined HAVE_DFT_BIGDFT
 integer :: ii,isc
 real(dp) :: rad_core,rad_cov,rad_long
 character(len=500) :: message
 integer :: mxpl,mxchg
 character(len=2) :: symbol
 integer :: nzatom, nelpsp, npspcode,ixc_
 real(dp) :: maxrad
 logical::exists
 real(dp) :: amu,rcov, rprb, ehomo, radfine
 real(dp) :: neleconf(6,0:3)
#endif

 
!arrays


! *************************************************************************
 
!==========================================================
 DBG_ENTER("COLL")


#if defined HAVE_DFT_BIGDFT
!Obtain physical quantities
 call eleconf(int(psps%znuclpsp(ipsp)), int(psps%zionpsp(ipsp)), symbol, rcov, rprb, &
& ehomo, neleconf, isc, mxpl, mxchg,amu)

!Obtain the HGH parameters by default from BigDFT
!the radius psppar(0,0) is needed for the definition of the mesh in BigDFT
!the remaining parameters are needed for the initial guess wavefunction in BigDFT

!!  Define a dummy value for rad_core
!rad_core=pawtab%rshp/4.d0 !dummy now
!psps%gth_params%psppar(0, 0, ipsp) = rad_core !(/ rloc, cc1, cc2, cc3, cc4, zero, zero /)

!I use the XC: Perdew, Burke & Ernzerhof  as default, since
!other XC potentials may not be in the BigDFT table.
 ixc_=1 
 call psp_from_data(symbol, nzatom, nelpsp, npspcode, ixc_, psps%gth_params%psppar(:,:,ipsp), exists)
 if(.not. exists) then
   write(message,'(a,a,a,a)')ch10,&
&   "psp7in : bug, Chemical element not found in BigDFT table",ch10,&
&   "Action: upgrade BigDFT table"
   call wrtout(ab_out,message,'COLL')
   MSG_BUG(message)
 end if
!assigning the radii by calculating physical parameters
 rad_long = one / sqrt(abs(two * ehomo))
 radfine  = 100.d0
 do ii = 0, 4, 1
   if (psps%gth_params%psppar(ii, 0, ipsp) /= zero) then
     radfine = min(radfine, psps%gth_params%psppar(ii, 0, ipsp))
   end if
 end do
 rad_core=radfine
 rad_cov=rcov
 write(message, '(9a)' ) '-', ch10,&
& '- psp7in : COMMENT -',ch10,&
& "-  HGH geometric informations have been computed:",ch10,&
& "-  These are needed for the initial guess",ch10,&
& "-  and to set the size of the mesh around the atoms"
 call wrtout(ab_out,message,'COLL')
 call wrtout(std_out,  message,'COLL')
 write(message, '(a,f12.7,a,f12.7,a,f12.7)' )&
& '  radii_cf(1)=', rad_long,'; radii_cf(2)=', rad_core, "; rad_cov=", rad_cov
 call wrtout(ab_out,message,'COLL')
 call wrtout(std_out,  message,'COLL')
!
 psps%gth_params%radii_cf(ipsp, :)  = (/ rad_long, rad_core,rad_core /)
 psps%gth_params%radii_cov(ipsp)    = rad_cov
 if (rad_core > tol12 .and. rad_long > tol12) then
   psps%gth_params%hasGeometry(ipsp) = .true.
 else
   psps%gth_params%hasGeometry(ipsp) = .false.
 end if
 psps%gth_params%semicore(ipsp)     = isc
!correct the coarse and the fine radius for projectors
 maxrad = zero
 do ii = 0, 4, 1
!  the maximum radii is useful only for projectors
   if (ii == 1) maxrad = zero
   if (psps%gth_params%psppar(ii, 0, ipsp) /= zero) then
     maxrad = max(maxrad, psps%gth_params%psppar(ii, 0, ipsp))
   end if
 end do
!For H, for instance maxrad=0, and the radii for projectors 
!(radii_cf(ipsp,3)) should  not be zero.
 if(abs(maxrad)>tol3) then
   psps%gth_params%radii_cf(ipsp,3)   = min(wvl_crmult * &
&   psps%gth_params%radii_cf(ipsp, 1), real(15.0, dp) * maxrad) / wvl_frmult
 else
!  psps%gth_params%radii_cf(ipsp,3)   = max(pawtab%rpaw /wvl_frmult,&
!  &     psps%gth_params%radii_cf(ipsp,2))
   psps%gth_params%radii_cf(ipsp,3)   = &
&   (psps%gth_params%radii_cf(ipsp, 1)+psps%gth_params%radii_cf(ipsp,2))/2.d0
 end if
 write(message, '(a,f12.7)' )&
& '  radii for projectors: radii_cf(3)=', psps%gth_params%radii_cf(ipsp,3)
 call wrtout(ab_out,message,'COLL')
 call wrtout(std_out,  message,'COLL')
#endif


 DBG_EXIT('COLL')

end subroutine psp7wvl2
!!***

