!{\src2tex{textfont=tt}}
!!****f* ABINIT/calc_sig_ppm_delta_clos_cd
!! NAME
!! calc_sig_ppm_delta_clos_cd
!!
!! FUNCTION
!! Calculation of the part of the matrix elements of the self-energy coming from
!! the sum over all the states in OPTIMAL GW
!! 
!! INPUTS
!! 
!! OUTPUT
!! 
!! PARENTS
!!      calc_sig_cd_eet
!!
!! CHILDREN
!!      spline,splint,xgemm,xgemv
!!
!! SOURCE
#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

subroutine calc_sig_ppm_delta_clos_cd(Er,npwc,nomega,ikbz,jkbz,qbzpg,omegame0k,omegame0lumo,ptwsq,epsm1_qbz, &
&                                     ket,gw_eet_scale,niter)

 use defs_basis
 use m_profiling
 use m_errors
 use m_screening,     only : epsilonm1_results

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'calc_sig_ppm_delta_clos_cd'
 use interfaces_70_gw, except_this_one => calc_sig_ppm_delta_clos_cd
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 type(Epsilonm1_results),intent(in) :: Er
 integer,intent(in) :: nomega,npwc,niter
 integer,intent(in) :: ikbz,jkbz
 real(dp),intent(in) :: gw_eet_scale
!arrays
 real(dp),intent(in) :: omegame0k(nomega),omegame0lumo(nomega)
 real(dp),intent(in) :: qbzpg(npwc)
 complex(gwpc),intent(in) :: ptwsq(npwc,npwc,niter+1)
 complex(gwpc),intent(in) :: epsm1_qbz(npwc,npwc,Er%nomega)
 complex(dpc),intent(inout) :: ket(nomega)

!Local variables-------------------------------
!scalars
 integer :: ig,igp,ios,io,iaux
 real(dp) :: bq
 complex(gwpc) :: num,delta
 logical :: ggpzero
!arrays
 real(dp), allocatable :: qpgsq(:)
 real(dp) :: omegame0k_tmp(nomega)
 real(dp) :: left(nomega),right(nomega)
 complex(gwpc) :: weight(Er%nomega_i+1,nomega)
 complex(dpc) :: omega_imag(Er%nomega_i+1)
 complex(dpc) :: domegaleft,domegaright

!*************************************************************************

 ABI_ALLOCATE(qpgsq,(npwc))

 do ig = 1, npwc
   qpgsq(ig)=half*qbzpg(ig)*qbzpg(ig)
 enddo

 ! Frequency mesh for integral along the imaginary axis.    
 omega_imag(1)=Er%omega(1)
 omega_imag(2:Er%nomega_i+1)=Er%omega(Er%nomega_r+1:Er%nomega)

 do ig = 1, npwc
   do igp = 1, npwc

     bq=half*(qpgsq(ig)+qpgsq(igp))
     if (niter==0) then
       delta = bq
     elseif(niter==1) then
       delta = bq+ptwsq(ig,igp,2)
     endif
     ggpzero=(ig==1.and.igp==1)
     if (ikbz==jkbz.and.ggpzero) delta = 0.0_gwp

     do ios = 1, nomega
       call check_delta_sigma(qpgsq(ig),qpgsq(igp),delta,omegame0k(ios),omegame0lumo(ios), &
&                             ig,igp,gw_eet_scale,niter)
       omegame0k_tmp(ios)=omegame0k(ios) - delta
       ! Avoid divergences in $\omega - \omega_s$.
       if (ABS(omegame0k_tmp(ios))<tol6) omegame0k_tmp(ios)=sign(tol6,omegame0k_tmp(ios))
     end do

     weight(1,:) = ATAN(-half*AIMAG(omega_imag(2))/REAL(omegame0k_tmp(:)))
     domegaleft  = (three*omega_imag(Er%nomega_i+1)-omega_imag(Er%nomega_i))
     domegaright = (omega_imag(Er%nomega_i+1)+omega_imag(Er%nomega_i))
     right(:)    = -AIMAG(omega_imag(Er%nomega_i+1)-omega_imag(Er%nomega_i))*REAL(omegame0k_tmp(:))
     left(:)     = quarter*AIMAG(domegaleft)*AIMAG(domegaright) & 
                        +REAL(omegame0k_tmp(:))*REAL(omegame0k_tmp(:))
     weight(Er%nomega_i+1,:) = ATAN(right(:)/left(:))
     do io=2,Er%nomega_i
       domegaleft  = (omega_imag(io  )+omega_imag(io-1))
       domegaright = (omega_imag(io+1)+omega_imag(io  ))
       right(:)    = -half*AIMAG(omega_imag(io+1)-omega_imag(io-1))*REAL(omegame0k_tmp(:))
       left(:)     = REAL(omegame0k_tmp(:))*REAL(omegame0k_tmp(:)) & 
        +quarter*AIMAG(domegaleft)*AIMAG(domegaright)
       weight(io,:) = ATAN(right(:)/left(:))
     end do

     do io = 1, Er%nomega_i+1
       if (io==1) then
         iaux = 1
       else
         iaux = Er%nomega_r+1
       endif
       num = piinv*epsm1_qbz(ig,igp,iaux)*ptwsq(ig,igp,1)
       do ios=1,nomega
         ket(ios) = ket(ios) + num*weight(io,ios)
       enddo
     end do

   enddo
 enddo

 ABI_DEALLOCATE(qpgsq)

end subroutine calc_sig_ppm_delta_clos_cd
!!***

!-----------------------------------------------------

!!****f* ABINIT/gw_eet_sigma_cd
!! NAME
!! gw_eet_sigma_cd
!!
!! FUNCTION
!! Wrapper routine for the calculation of the matrix elements of Sigma using GW_OPTIMAL
!!
!! INPUTS
!!
!! OUTPUT
!!
!! PARENTS
!!      calc_sigc_me
!!
!! CHILDREN
!!      spline,splint,xgemm,xgemv
!!
!! SOURCE
#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

subroutine gw_eet_sigma_cd(Sigp,Sr,Dtset,Cryst,Wfs,Kmesh,Qmesh,Gsph_Max,Gsph_c,Psps,Vcp,QP_BSt,Er, &
&                       isppol,iq_bz,ik_bz,jk_bz,ik_ibz,jk_ibz,itim_q,isym_q,iq_ibz,tabr_ki, &
&                       tabr_kj,spinrot_ki,spinrot_kj,ph_mkit,ph_mkjt,nfftot_gw,ngfft_gw, &
&                       use_padfft,igfftcg0,gw_gbound,gw_mgfft,ib1,ib2,nomega_tot,nomega_sigc, &
&                       fact_sp,nspinor,sigcme_tmp,sigc,nbhomo,tim_fourdp,wtqp,wtqm, &
&                       extrapolar_distrb,can_symmetrize,epsm1_qbz,npoles_missing)

 use defs_basis
 use m_gwdefs
 use defs_datatypes
 use defs_abitypes
 use m_crystal
 use m_bz_mesh
 use m_gsphere
 use m_wfs
 use m_commutator_vkbr
 use m_ppmodel
 use m_profiling
 use m_errors

 use m_geometry,      only : normv, vdotw
 use m_vcoul,         only : vcoul_t
 use m_sigma_results, only : sigma_results
 use m_screening,     only : epsilonm1_results

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'gw_eet_sigma_cd'
 use interfaces_70_gw, except_this_one => gw_eet_sigma_cd
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: nfftot_gw,ngfft_gw(18),tim_fourdp,use_padfft,gw_mgfft
 integer,intent(in) :: isppol,itim_q,isym_q,iq_ibz,nspinor
 integer,intent(in) :: nomega_tot,nomega_sigc
 integer,intent(in) :: wtqp,wtqm
 integer,intent(in) :: iq_bz,ik_bz,jk_bz,ik_ibz,jk_ibz,ib1,ib2
 integer,intent(inout) :: npoles_missing(ib1:ib2)
 integer,intent(out) :: nbhomo
 real(dp),intent(in) :: fact_sp
 complex(dpc),intent(in) ::  ph_mkit,ph_mkjt
 type(Crystal_structure),intent(in) :: Cryst
 type(BZ_mesh_type),intent(in) :: Kmesh,Qmesh
 type(vcoul_t),intent(in) :: Vcp
 type(Gvectors_type),intent(in) :: Gsph_Max
 type(Gvectors_type),intent(in) :: Gsph_c
 type(Dataset_type),intent(in) :: Dtset
 type(Pseudopotential_type),intent(in) :: Psps
 type(Sigma_parameters),intent(in) :: Sigp
 type(Sigma_results),intent(in) :: Sr
 type(wfs_descriptor),intent(inout) :: Wfs
 type(Bandstructure_type),intent(in) :: QP_BSt
 type(Epsilonm1_results),intent(in) :: Er
!arrays
 integer,intent(in) :: igfftcg0(Sigp%npwc)
 integer,intent(in) :: gw_gbound(2*gw_mgfft+8,2*use_padfft)
 integer,intent(in) :: tabr_ki(nfftot_gw),tabr_kj(nfftot_gw)
 integer,intent(in) :: extrapolar_distrb(ib1:ib2,ib1:ib2,Kmesh%nbz,Wfs%nsppol)
 real(dp),intent(in) :: spinrot_ki(4),spinrot_kj(4)
 complex(gwpc),intent(in) :: epsm1_qbz(Sigp%npwc,Sigp%npwc,Er%nomega)
 complex(dpc),intent(inout) :: sigcme_tmp(nomega_sigc,ib1:ib2,ib1:ib2,Sigp%nsppol*Sigp%nsig_ab)
 complex(dpc),intent(inout) :: sigc(2,nomega_sigc,ib1:ib2,ib1:ib2,Sigp%nsppol*Sigp%nsig_ab)
 logical,intent(in) :: can_symmetrize(Wfs%nsppol)

!Local variables ------------------------------
!scalars
 integer :: io,kb,jb
 integer :: niter,nptwg,iter,nbmax
 integer :: ig,igp,ib,ibv
 integer :: isym_kgw,isym_ki,iik,jik
!arrays
 integer :: fnlloc(Cryst%ntypat,2),fnlmax(Cryst%ntypat)
 real(dp),pointer :: qp_ene(:,:,:),qp_occ(:,:,:)
 real(dp),allocatable :: qplg(:,:)
 real(dp),allocatable :: kplqg(:)
 real(dp),allocatable :: omegame0k(:),omegame0lumo(:)
 real(dp),allocatable :: qbzpg(:) 
 complex(gwpc),allocatable :: vc_sqrt_qbz(:)
 complex(gwpc),allocatable :: fnlkr(:,:,:)
 complex(gwpc),allocatable :: fnlkpr(:,:,:)
 complex(gwpc),allocatable :: wfr1(:,:)
 complex(gwpc),allocatable :: mtwk(:,:)
 complex(gwpc),allocatable :: mtwkp(:,:)
 complex(gwpc),allocatable :: ptwsq(:,:,:)
 complex(dpc) :: sigctmp(nomega_sigc)

!************************************************************************

 qp_ene => QP_BSt%eig(:,:,:)
 qp_occ => QP_BSt%occ(:,:,:)

 !@Arjan:
 ! Gsph_c gamma-centered sphere for W and thus Sigma_x
 ! Gsph_Max gamma-centered sphere with gvec(3,npwvec) where Sigp%npwvec=MAX(Sigp%npwwfn,Sigp%npwx)
 ! it is used for the wavefunctions but it will be REMOVED when we switch to k-centered
 ! G-spheres for the wavefunctions

 !call init_gsphere(Gsph_Max,Cryst,Sigp%npwvec,gvec=gvec_kss)

 ! min and Max band indeces for GW corrections (for this k-point)

 nbhomo=1
 do ib = 2, Sigp%nbnds 
   if (fact_sp*qp_occ(ib,jk_ibz,isppol)<GW_TOL_DOCC) exit
   nbhomo=nbhomo+1
 enddo
 nbmax=max(nbhomo,Dtset%gw_eet_nband)

 !allocate(val_idx(QP_BSt%nkpt,QP_BSt%nsppol))
 !val_idx = get_valence_idx(QP_BSt,tol3)
 !do isppol=1,nsppol
 ! nbhomo(isppol) = val_idx(1,isppol)
 ! ltest = ALL(val_idx(:,isppol))==val_idx(1,isppol),
 ! ABI_CHECK(ltest,"Optimal GW + metals not coded")
 !end do
 !deallocate(val_idx)

 niter = Dtset%gw_eet
 nptwg=1
 do iter = 1, niter
   nptwg=nptwg+mod(iter,2)
 enddo

 !call init_iskg_tabs(ISkg,Cryst,k_tim,k_sym,Wfd%ecut,istwf_k,npw_k,k_ibz,kg_k,Cryst%gmet,Wfd%ngfft,ierr)
 !ABI_CHECK(ierr==0," init_iskg_tabs returned ierr/=0")

 ABI_ALLOCATE(vc_sqrt_qbz,(Sigp%npwc))
 do ig=1,Sigp%npwc
   vc_sqrt_qbz(Gsph_c%rottb(ig,itim_q,isym_q))=Vcp%vc_sqrt(ig,iq_ibz)
 end do

 ABI_ALLOCATE(omegame0k,(nomega_tot))
 ABI_ALLOCATE(omegame0lumo,(nomega_tot))
 ABI_ALLOCATE(qplg,(Sigp%npwc,3))
 ABI_ALLOCATE(kplqg,(Sigp%npwc))
 ABI_ALLOCATE(qbzpg,(Sigp%npwc))

 isym_kgw = Kmesh%tabo(jk_bz)
 jik = (3-Kmesh%tabi(jk_bz))/2

 isym_ki = Kmesh%tabo(ik_bz)
 iik = (3-Kmesh%tabi(ik_bz))/2

 do ig=1,Sigp%npwc
   qplg(ig,:) = Qmesh%bz(:,iq_bz) + Gsph_c%gvec(:,ig)
   kplqg(ig) = -vdotw(Kmesh%bz(:,jk_bz),qplg(ig,:),Cryst%gmet,"G")
   qbzpg(ig) = normv(qplg(ig,:),Cryst%gmet,"G")
 end do

 if (niter>0.and.Dtset%gw_eet_inclvkb==1) then

   ABI_ALLOCATE(fnlkr,(Wfs%nfftot*nspinor,Psps%mpsang*Psps%mpsang,Cryst%natom))
   ABI_ALLOCATE(fnlkpr,(Wfs%nfftot*nspinor,Psps%mpsang*Psps%mpsang,Cryst%natom))
   ABI_ALLOCATE(mtwk,(Wfs%nfftot*nspinor,nbmax))
   ABI_ALLOCATE(mtwkp,(Wfs%nfftot*nspinor,ib1:ib2))

   call gw_eet_sigma_vkb(Sigp,Cryst,Wfs,Kmesh,Psps,isppol,ik_ibz,jk_ibz,ib1,ib2,nspinor, &
&                        tim_fourdp,nbmax,fnlkr,fnlkpr,mtwk,mtwkp,fnlloc,fnlmax)
 endif

 ABI_ALLOCATE(wfr1,(Wfs%nfftot*nspinor,nbmax))

 do ibv = 1, nbmax
   call wfd_get_ur(Wfs,ibv,ik_ibz,isppol,wfr1(:,ibv))
 enddo

 do jb = ib1,ib2

   do kb = ib1,ib2

     if (Sigp%gwcalctyp/=28.and.kb/=jb) CYCLE

     if (extrapolar_distrb(jb,kb,ik_bz,isppol)/=Wfs%my_rank) CYCLE

     ABI_ALLOCATE(ptwsq,(Sigp%npwc,Sigp%npwc,niter+1))

     do io=1,Sr%nomega4sd
       omegame0k(io)  = real(Sr%omega4sd(kb,jk_ibz,io,isppol)) - qp_ene(kb,jk_ibz,isppol)
       omegame0lumo(io)= real(Sr%omega4sd(kb,jk_ibz,io,isppol)) - qp_ene(nbmax+1,ik_ibz,isppol)
     end do

     sigctmp=czero_gw

     if (niter==0.or.Dtset%gw_eet_inclvkb==0) then

       call fft4eet_sig_cd(Sigp,Cryst,Wfs,Kmesh,Gsph_c,Sr,Er,nbhomo,nbmax,nomega_tot,isppol,nfftot_gw, &
&                   ngfft_gw,use_padfft,igfftcg0,gw_gbound,gw_mgfft,iik,tabr_ki,ph_mkit,spinrot_ki, &
&                   ik_ibz,jk_ibz,isym_kgw,jik,tabr_kj,ph_mkjt,spinrot_kj,Gsph_Max, &
&                   nspinor,tim_fourdp,wfr1,vc_sqrt_qbz,Vcp%i_sz,kb,qplg,kplqg,niter, &
&                   ptwsq,ik_bz,jk_bz,npoles_missing(kb),epsm1_qbz,sigctmp)

     else

       call fft4eet_sig_kb_cd(Sigp,Cryst,Wfs,Kmesh,Gsph_c,Psps,Sr,Er,nbhomo,nbmax,nomega_tot,isppol,nfftot_gw, &
&                   ngfft_gw,use_padfft,igfftcg0,gw_gbound,gw_mgfft,iik,tabr_ki,ph_mkit,spinrot_ki, &
&                   ik_ibz,jk_ibz,isym_kgw,jik,tabr_kj,ph_mkjt,spinrot_kj,Gsph_Max, &
&                   nspinor,tim_fourdp,fnlloc,fnlmax,fnlkr,mtwk,mtwkp(:,kb),wfr1,vc_sqrt_qbz,Vcp%i_sz, &
&                   kb,qplg,kplqg,niter,ptwsq,ik_bz,jk_bz,npoles_missing(kb),epsm1_qbz,sigctmp)

     endif

     call calc_delta_ppm(Sigp,ptwsq,niter)

     do ig = 1, Sigp%npwc
       do igp = 1, Sigp%npwc
         if (ik_bz==jk_bz) then
           if (ig/=1.and.igp/=1) then
             ptwsq(ig,igp,1)= ptwsq(ig,igp,1)*vc_sqrt_qbz(ig)*vc_sqrt_qbz(igp)
           else
             if (jb/=kb.or.kb<=nbmax.or.jb<=nbmax) then
               ptwsq(ig,igp,1)=(0.0,0.0)
             else
               if (ig==1.and.igp==1) then
                 ptwsq(ig,igp,1) = cmplx(Vcp%i_sz,0.0_gwp)
               elseif (ig==1.and.igp/=1) then
                 ptwsq(ig,igp,1) = cmplx(sqrt(Vcp%i_sz),0.0_gwp)*ptwsq(ig,igp,1)*vc_sqrt_qbz(igp)
               elseif (igp==1.and.ig/=1) then
                 ptwsq(ig,igp,1) = cmplx(sqrt(Vcp%i_sz),0.0_gwp)*ptwsq(ig,igp,1)*vc_sqrt_qbz(ig)
               else
                 ptwsq(ig,igp,1)= ptwsq(ig,igp,1)*vc_sqrt_qbz(ig)*vc_sqrt_qbz(igp)
               endif
             endif
           endif
         else
           ptwsq(ig,igp,1)= ptwsq(ig,igp,1)*vc_sqrt_qbz(ig)*vc_sqrt_qbz(igp)
         endif
       enddo
     enddo

     call calc_sig_ppm_delta_clos_cd(Er,Sigp%npwc,nomega_tot,ik_bz,jk_bz,qbzpg,omegame0k,omegame0lumo, &
&                                    ptwsq,epsm1_qbz,sigctmp,Dtset%gw_eet_scale,niter)

     if (can_symmetrize(isppol)) then
       sigcme_tmp(:,jb,kb,isppol)=sigcme_tmp(:,jb,kb,isppol) + &
         (wtqp+wtqm)*DBLE(sigctmp(:)) + (wtqp-wtqm)*j_gw*AIMAG(sigctmp(:))
       sigc(1,:,jb,kb,isppol)=sigc(1,:,jb,kb,isppol) + wtqp*      sigctmp(:)
       sigc(2,:,jb,kb,isppol)=sigc(2,:,jb,kb,isppol) + wtqm*CONJG(sigctmp(:))
     else
       sigcme_tmp(:,jb,kb,isppol)=sigcme_tmp(:,jb,kb,isppol)+sigctmp(:)
     endif

     ABI_DEALLOCATE(ptwsq)

   enddo
 enddo

 if (niter>0.and.Dtset%gw_eet_inclvkb==1) then
   ABI_DEALLOCATE(mtwk)
   ABI_DEALLOCATE(mtwkp)
   ABI_DEALLOCATE(fnlkr)
   ABI_DEALLOCATE(fnlkpr)
 endif

 ABI_DEALLOCATE(wfr1)
 ABI_DEALLOCATE(vc_sqrt_qbz)
 ABI_DEALLOCATE(omegame0k)
 ABI_DEALLOCATE(omegame0lumo)
 ABI_DEALLOCATE(qplg)
 ABI_DEALLOCATE(kplqg)
 ABI_DEALLOCATE(qbzpg)

end subroutine gw_eet_sigma_cd
!!***
 
!----------------------------------------------------------------------

!!****f* ABINIT/fft4eet_sig_cd
!! NAME                  
!! fft4eet_sig_cd
!!
!! FUNCTION
!! (to be provided)
!!
!! INPUTS
!!
!! OUTPUT
!!
!! PARENTS
!!      calc_sig_cd_eet
!!
!! CHILDREN
!!      spline,splint,xgemm,xgemv
!!
!! SOURCE
#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

subroutine fft4eet_sig_cd(Sigp,Cryst,Wfs,Kmesh,Gsph_c,Sr,Er,nbhomo,nbmax,nomega,is,nfftot_gw,ngfft_gw, &
&                  use_padfft,igfftepsG0,gw_gbound,gw_mgfft,itim_k,tabr_k,ph_mkt,spinrot_k, &
&                  ik_ibz,ikmq_ibz,isym_kmq,itim_kmq,tabr_kmq,ph_mkmqt,spinrot_kmq,Gsph_max, &
&                  nspinor,tim_fourdp,wfr1,vc_sqrt_qbz,i_sz,kb,qplg,kplqg,niter, &
&                  ptwsq,ik_bz,ikmq_bz,npoles_missing,epsm1q,sigmac)

 use defs_basis
 use defs_datatypes
 use defs_abitypes
 use m_gwdefs
 use m_crystal
 use m_bz_mesh
 use m_gsphere
 use m_wfs
 use m_profiling
 use m_errors

 use m_geometry,      only : vdotw
 use m_vcoul,         only : vcoul_t
 use m_oscillators,   only : rho_tw_g, calc_wfwfg
 use m_sigma_results, only : sigma_results
 use m_blas,          only : xgerc
 use m_screening,     only : epsilonm1_results

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fft4eet_sig_cd'
 use interfaces_14_hidewrite
 use interfaces_70_gw, except_this_one => fft4eet_sig_cd
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 type(Crystal_structure),intent(in) :: Cryst
 type(Sigma_parameters),intent(in) :: Sigp
 type(wfs_descriptor),intent(inout) :: Wfs
 type(Sigma_results),intent(in) :: Sr
 type(Gvectors_type),intent(in) :: Gsph_c,Gsph_max
 type(BZ_mesh_type),intent(in) :: Kmesh
 type(Epsilonm1_results),intent(in) :: Er
 integer,intent(in) :: nbhomo,nbmax,is,kb,niter,nomega
 integer,intent(in) :: nfftot_gw,ngfft_gw(18),nspinor,tim_fourdp,use_padfft,gw_mgfft
 integer,intent(in) :: ik_bz,ikmq_bz,ik_ibz,ikmq_ibz
 integer,intent(in) :: isym_kmq,itim_k,itim_kmq
 integer,intent(in) :: tabr_k(nfftot_gw),tabr_kmq(nfftot_gw)
 integer,intent(in) :: igfftepsG0(Sigp%npwc)
 integer,intent(in) :: gw_gbound(2*gw_mgfft+8,2*use_padfft)
 integer,intent(inout) :: npoles_missing
 real(dp),intent(in) :: spinrot_k(4),spinrot_kmq(4)
 real(dp),intent(in) :: qplg(Sigp%npwc,3),kplqg(Sigp%npwc)
 real(dp),intent(in) :: i_sz
 complex(dpc),intent(in) :: ph_mkmqt,ph_mkt
!arrays
 complex(gwpc),intent(in) :: wfr1(Wfs%nfftot*nspinor,nbmax)
 complex(gwpc),intent(in) :: vc_sqrt_qbz(Sigp%npwc)
 complex(gwpc),intent(in) :: epsm1q(Sigp%npwc,Sigp%npwc,Er%nomega)
 complex(gwpc),intent(out) :: ptwsq(Sigp%npwc,Sigp%npwc,niter+1)
 complex(dpc),intent(inout) :: sigmac(nomega)

!Local variables-------------------------------
!scalars
 integer :: istwf_kmq,nbz,npw_kmq,ibv,ig,igp,i,j
 integer :: ig4,ig4x,ig4y,ig4z,outofbox
 integer,save :: enough=0
 complex(gwpc) :: minusone
 character(len=500) :: msg
!arrays
 integer :: gmgp(3)
 integer,pointer :: gbound_kmq(:,:),kg_kmq(:,:),igfft_kmq(:)
 integer,parameter :: ndat1=1
 complex(dpc) :: drhaux(3)
 complex(dpc) :: paux(3)
 complex(gwpc),allocatable :: wfr2(:)
 complex(gwpc),allocatable :: rhotwg(:,:)
 complex(gwpc),allocatable :: drhotwg(:,:,:)
 complex(gwpc),allocatable :: wfwfg(:)
 complex(gwpc),allocatable :: dwfwfg(:,:)
 complex(gwpc),allocatable :: ddwfwfg(:,:,:)
 complex(gwpc),allocatable :: dwfr(:,:)
 complex(gwpc),allocatable :: gwfg(:)
 complex(gwpc),allocatable :: cauxg(:,:)
 !integer :: igbz
 !complex(dpc) :: ph_Gt

!************************************************************************

 ABI_UNUSED((/isym_kmq,tim_fourdp/))

!Dummy statement, to keep Kmesh arg
 nbz=Kmesh%nbz

 gbound_kmq => Wfs%Kdata(ikmq_ibz)%gbound
 igfft_kmq  => Wfs%Kdata(ikmq_ibz)%igfft0
 kg_kmq     => Wfs%Kdata(ikmq_ibz)%kg_k
 istwf_kmq  = Wfs%istwfk(ikmq_ibz)
 npw_kmq    = Wfs%npwarr(ikmq_ibz)

 ABI_ALLOCATE(wfr2,(Wfs%nfftot*nspinor))

 ABI_ALLOCATE(rhotwg,(Sigp%npwc*nspinor**2,nbmax))
 ABI_ALLOCATE(wfwfg,(nfftot_gw*nspinor**2))

 if (niter>0) then
   ABI_ALLOCATE(drhotwg,(Sigp%npwc*nspinor**2,nbmax,3))
   ABI_ALLOCATE(dwfwfg,(nfftot_gw*nspinor**2,3))
   ABI_ALLOCATE(gwfg,(npw_kmq))
   ABI_ALLOCATE(dwfr,(Wfs%nfftot*nspinor,3))
 endif

 if (niter>1) then
   ABI_ALLOCATE(ddwfwfg,(nfftot_gw*nspinor**2,3,3))
 endif

 call wfd_get_ur(Wfs,kb,ikmq_ibz,is,wfr2)
 call calc_wfwfg(tabr_kmq,itim_kmq,nfftot_gw,ngfft_gw,wfr2,wfr2,wfwfg(:))

 do ibv = 1, nbmax
   call rho_tw_g(nspinor,Sigp%npwc,nfftot_gw,ndat1,ngfft_gw,1,use_padfft,igfftepsG0,gw_gbound, &
&                wfr1(:,ibv),itim_k,tabr_k,ph_mkt,spinrot_k,wfr2,itim_kmq,tabr_kmq,ph_mkmqt, &
&                spinrot_kmq,nspinor,rhotwg(:,ibv))

   if (ibv>nbhomo) then
      call calc_corr_sig_cd(Sr,Sigp%npwc,Sigp%npwx,ik_bz,ikmq_bz,ik_ibz,ikmq_ibz,ibv,kb,is,nomega, &
&                           Er%nomega,Er%nomega_r,Er%nomega_i,rhotwg(:,ibv), &
&                           Er%omega,epsm1q,npoles_missing,i_sz,vc_sqrt_qbz,sigmac)
   endif
 enddo

 if (niter>0) then

   call wfd_dur_isk(Wfs,Cryst,Kmesh,kb,ikmq_bz,is,npw_kmq,Gsph_max%rottbm1,dwfr)  !,ISkg,ik_ibz

   do ibv = 1, nbmax
     do i = 1, 3
       call drho_tw_g(nspinor,Sigp%npwc,nfftot_gw,ngfft_gw,1,use_padfft,igfftepsG0,gw_gbound,&
&                     wfr1(:,ibv),itim_k,tabr_k,ph_mkt,dwfr(:,i), &
&                     nspinor,drhotwg(:,ibv,i))
     enddo
   enddo

   do i = 1, 3
     call calc_dwfwfg(tabr_kmq,itim_kmq,nfftot_gw,ngfft_gw,ph_mkmqt,wfr2,dwfr(:,i),dwfwfg(:,i))
     if (niter>1) then
       do j = 1, 3
         call calc_ddwfwfg(itim_kmq,nfftot_gw,ngfft_gw,dwfr(:,i),dwfr(:,j),ddwfwfg(:,i,j))
       enddo
     endif
   enddo

   !ABI_DEALLOCATE(wfr2)
   ABI_DEALLOCATE(gwfg)
   ABI_DEALLOCATE(dwfr)
   ABI_ALLOCATE(cauxg,(Sigp%npwc,nbmax))

   cauxg(:,:)=(0.0,0.0)
   do ibv = 1, nbmax
     do ig = 1, Sigp%npwc
       drhaux(:)=cmplx(real(drhotwg(ig,ibv,:)),aimag(drhotwg(ig,ibv,:)))
       cauxg(ig,ibv)=vdotw(qplg(ig,:),drhaux,Cryst%gmet,"G")
     enddo
   enddo

 endif
 ABI_DEALLOCATE(wfr2)

 ptwsq(:,:,:)=(0.0,0.0)
 outofbox=0
 do igp=1,Sigp%npwc
   do ig=1,Sigp%npwc
     gmgp(:)=Gsph_c%gvec(:,igp)-Gsph_c%gvec(:,ig)
     if (ANY(gmgp(:)>ngfft_gw(1:3)/2) .or. ANY(gmgp(:)<-(ngfft_gw(1:3)-1)/2)) then
       outofbox = outofbox+1; CYCLE
     end if
     ig4x= modulo(gmgp(1),ngfft_gw(1))
     ig4y= modulo(gmgp(2),ngfft_gw(2))
     ig4z= modulo(gmgp(3),ngfft_gw(3))
     ig4= 1+ig4x+ig4y*ngfft_gw(1)+ig4z*ngfft_gw(1)*ngfft_gw(2)
     if (igp>=ig) then
       ptwsq(ig,igp,1)=wfwfg(ig4)
     endif
     if (niter>0) then
       drhaux(:)=cmplx(real(dwfwfg(ig4,:)),aimag(dwfwfg(ig4,:)))
       ptwsq(ig,igp,2)=ptwsq(ig,igp,2) + vdotw(qplg(igp,:),drhaux,Cryst%gmet,"G")
       if (niter>1.and.igp>=ig) then
         paux(:)=czero
         do i = 1, 3
           drhaux(:)=cmplx(real(ddwfwfg(ig4,:,i)),aimag(ddwfwfg(ig4,:,i)))
           paux(i)=paux(i) + vdotw(qplg(ig,:),drhaux,Cryst%gmet,"G")
         enddo
         ptwsq(ig,igp,3)=ptwsq(ig,igp,3) + vdotw(qplg(igp,:),paux,Cryst%gmet,"G")
       endif
     endif
   enddo
 enddo

 if (outofbox/=0) then
   enough=enough+1
   if (enough<=10) then
     write(msg,'(a,i5)')' Number of G1-G2 pairs outside the G-sphere for Wfns = ',outofbox
     MSG_WARNING(msg)
     if (enough==10) then
       write(msg,'(a)')' ========== Stop writing Warnings =========='
       call wrtout(std_out,msg,'COLL')
     end if
   end if
 end if

 do ibv = 1, nbmax
   do igp=1,Sigp%npwc
     do ig=1,igp
       ptwsq(ig,igp,1)=ptwsq(ig,igp,1)-conjg(rhotwg(ig,ibv))*rhotwg(igp,ibv)
     enddo
   end do
 end do

 do igp = 1, Sigp%npwc
   do ig = igp+1, Sigp%npwc
     ptwsq(ig,igp,1)=conjg(ptwsq(igp,ig,1))
   enddo
 enddo

 if (niter>0) then
   minusone=(-1.,0.)
   do ibv = 1, nbmax
     call XGERC(Sigp%npwc,Sigp%npwc,minusone,conjg(rhotwg(:,ibv)),1,conjg(cauxg(:,ibv)),1,ptwsq(:,:,2),Sigp%npwc)
   enddo
   do igp=1,Sigp%npwc
     do ig=1,Sigp%npwc
       ptwsq(ig,igp,2)=ptwsq(ig,igp,2)+ptwsq(ig,igp,1)*kplqg(igp)
     end do !igp
   end do !ig
 endif

 if (niter>1) then

   do ibv = 1, nbmax
     do igp=1,Sigp%npwc
       do ig=1,igp
         ptwsq(ig,igp,3)=ptwsq(ig,igp,3)-conjg(cauxg(ig,ibv))*cauxg(igp,ibv)
       enddo
     enddo
   enddo
   do igp=1,Sigp%npwc
     do ig=1,igp
       ptwsq(ig,igp,3)=ptwsq(ig,igp,3)+kplqg(igp)*ptwsq(ig,igp,2)+kplqg(ig)*conjg(ptwsq(igp,ig,2))&
&                                     -kplqg(ig)*ptwsq(ig,igp,1)*kplqg(igp)
     end do !igp
   end do !ig

   do igp = 1, Sigp%npwc
     do ig = igp+1, Sigp%npwc
       ptwsq(ig,igp,3)=conjg(ptwsq(igp,ig,3))
     enddo
   enddo

 endif

 ABI_DEALLOCATE(rhotwg)
 ABI_DEALLOCATE(wfwfg)
 if (niter>0) then
   ABI_DEALLOCATE(cauxg)
   ABI_DEALLOCATE(drhotwg)
   ABI_DEALLOCATE(dwfwfg)
 endif
 if (niter>1) then
   ABI_DEALLOCATE(ddwfwfg)
 endif

end subroutine fft4eet_sig_cd
!!***

!----------------------------------------------------------------------

!!****f* ABINIT/fft4eet_sig_kb_cd
!! NAME                  
!! fft4eet_sig_kb_cd
!!
!! FUNCTION
!! (to be provided)
!!
!! INPUTS
!!
!! OUTPUT
!!
!! PARENTS
!!      calc_sig_cd_eet
!!
!! CHILDREN
!!      spline,splint,xgemm,xgemv
!!
!! SOURCE
#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

subroutine fft4eet_sig_kb_cd(Sigp,Cryst,Wfs,Kmesh,Gsph_c,Psps,Sr,Er,nbhomo,nbmax,nomega,is,nfftot_gw,ngfft_gw, &
&                  use_padfft,igfftepsG0,gw_gbound,gw_mgfft,itim_k,tabr_k,ph_mkt,spinrot_k, &
&                  ik_ibz,ikmq_ibz,isym_kmq,itim_kmq,tabr_kmq,ph_mkmqt,spinrot_kmq,Gsph_max, &
&                  nspinor,tim_fourdp,fnlloc,fnlmax,fnlkr,mtwk,mtwkp,wfr1, &
&                  vc_sqrt_qbz,i_sz,kb,qplg,kplqg,niter,ptwsq,ik_bz,ikmq_bz,npoles_missing,epsm1q,sigmac)

 use m_profiling

 use defs_basis
 use m_gwdefs
 use defs_datatypes
 use defs_abitypes
 use m_crystal
 use m_bz_mesh
 use m_gsphere
 use m_wfs
 use m_commutator_vkbr
 use m_errors

 use m_geometry,      only : vdotw
 use m_vcoul,         only : vcoul_t
 use m_oscillators,   only : rho_tw_g, calc_wfwfg
 use m_sigma_results, only : sigma_results
 use m_blas,          only : xgerc
 use m_screening,     only : epsilonm1_results

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fft4eet_sig_kb_cd'
 use interfaces_14_hidewrite
 use interfaces_70_gw, except_this_one => fft4eet_sig_kb_cd
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 type(Crystal_structure),intent(in) :: Cryst
 type(BZ_mesh_type),intent(in) :: Kmesh
 type(Pseudopotential_type),intent(in) :: Psps
 type(Sigma_parameters),intent(in) :: Sigp
 type(wfs_descriptor),intent(inout) :: Wfs
 !type(kb_potential) :: KBff_kmq_ibz
 type(Sigma_results),intent(in) :: Sr
 type(Gvectors_type),intent(in) :: Gsph_c, Gsph_max
 type(Epsilonm1_results),intent(in) :: Er

 integer,intent(in) :: nbhomo,nbmax,is,kb,niter,nomega
 integer,intent(in) :: nfftot_gw,ngfft_gw(18),nspinor,tim_fourdp,use_padfft,gw_mgfft
 integer,intent(in) :: ik_bz,ikmq_bz,ik_ibz,ikmq_ibz
 integer,intent(in) :: isym_kmq,itim_k,itim_kmq
 integer,intent(in) :: tabr_k(nfftot_gw),tabr_kmq(nfftot_gw)
 integer,intent(in) :: igfftepsG0(Sigp%npwc)
 integer,intent(in) :: gw_gbound(2*gw_mgfft+8,2*use_padfft)
 integer,intent(in) :: fnlloc(Cryst%ntypat,2)
 integer,intent(in) :: fnlmax(Cryst%ntypat)
 integer,intent(inout) :: npoles_missing
 real(dp),intent(in) :: spinrot_k(4),spinrot_kmq(4)
 real(dp),intent(in) :: qplg(Sigp%npwc,3),kplqg(Sigp%npwc)
 real(dp),intent(in) :: i_sz
 complex(dpc),intent(in) :: ph_mkmqt,ph_mkt

 complex(gwpc),intent(in) :: fnlkr(Wfs%nfftot*nspinor,Psps%mpsang*Psps%mpsang,Cryst%natom)
 complex(gwpc),intent(in) :: mtwk(Wfs%nfftot*nspinor,nbmax)
 complex(gwpc),intent(in) :: mtwkp(Wfs%nfftot*nspinor)

 complex(gwpc),intent(in) :: wfr1(Wfs%nfftot*nspinor,nbmax)

 complex(gwpc),intent(in) :: vc_sqrt_qbz(Sigp%npwc)
 complex(gwpc),intent(in) :: epsm1q(Sigp%npwc,Sigp%npwc,Er%nomega)

 complex(gwpc),intent(out) :: ptwsq(Sigp%npwc,Sigp%npwc,niter+1)
 complex(dpc),intent(inout) :: sigmac(nomega)

!Local variables-------------------------------
!scalars
 integer :: istwf_kmq,npw_kmq,i,j,ibv,ilm,iat,ig,igp 
 !integer :: igbz,iat2,ilm2
 integer :: ig4,ig4x,ig4y,ig4z
 integer :: ig5,ig5x,ig5y,ig5z,nlx,outofbox
 integer,save :: enough=0
 complex(gwpc) :: minusone
 !complex(dpc) :: ph_Gt
 character(len=500) :: msg
!arrays
 integer,pointer :: gbound_kmq(:,:),kg_kmq(:,:),igfft_kmq(:)
 integer :: gmgp(3)
 integer,parameter :: ndat1=1
 complex(gwpc),allocatable :: wfr2(:)
 complex(gwpc),allocatable :: rhotwg(:,:)
 complex(gwpc),allocatable :: drhotwg(:,:,:)
 complex(gwpc),allocatable :: fnltwg(:,:,:)
 complex(gwpc),allocatable :: fnltwg2(:,:)
 complex(gwpc),allocatable :: fnltwg3(:,:)
 complex(gwpc),allocatable :: kns(:,:,:)
 complex(gwpc),allocatable :: wfwfg(:)
 complex(gwpc),allocatable :: dwfwfg(:,:)
 complex(gwpc),allocatable :: ddwfwfg(:,:,:)
 complex(gwpc),allocatable :: fnlwfg(:)
 complex(gwpc),allocatable :: fkdwfg(:,:)
 complex(gwpc),allocatable :: fdrhotwg(:,:,:,:)
 complex(gwpc),allocatable :: lnkp(:)
 complex(gwpc),allocatable :: dwfr(:,:)
 complex(gwpc),allocatable :: gwfg(:)
 complex(gwpc),allocatable :: cauxg(:,:)
 complex(gwpc),allocatable :: ff(:,:,:,:)
 complex(gwpc),allocatable :: vzn(:,:,:)
 complex(gwpc),allocatable :: paux(:,:)
 complex(dpc) :: drhaux(3)
 complex(dpc) :: paux2(3)

!************************************************************************

 ABI_UNUSED((/isym_kmq,tim_fourdp/))

 gbound_kmq => Wfs%Kdata(ikmq_ibz)%gbound
 igfft_kmq  => Wfs%Kdata(ikmq_ibz)%igfft0
 kg_kmq     => Wfs%Kdata(ikmq_ibz)%kg_k
 istwf_kmq  = Wfs%istwfk(ikmq_ibz)
 npw_kmq    = Wfs%npwarr(ikmq_ibz)

 nlx = min(Psps%mpsang,4)

 ABI_ALLOCATE(wfr2,(Wfs%nfftot*nspinor))
 ABI_ALLOCATE(rhotwg,(Sigp%npwc*nspinor**2,nbmax))
 ABI_ALLOCATE(wfwfg,(nfftot_gw*nspinor**2))

 if (niter>0) then
   ABI_ALLOCATE(drhotwg,(Sigp%npwc*nspinor**2,nbmax,3))
   ABI_ALLOCATE(dwfwfg,(nfftot_gw*nspinor**2,3))
   ABI_ALLOCATE(gwfg,(npw_kmq))
   ABI_ALLOCATE(dwfr,(Wfs%nfftot*nspinor,3))
   ABI_ALLOCATE(fnltwg,(Sigp%npwc*nspinor**2,Psps%mpsang*Psps%mpsang,Cryst%natom))
   ABI_ALLOCATE(fnlwfg,(nfftot_gw*nspinor**2))
   ABI_ALLOCATE(fnltwg2,(Sigp%npwc*nspinor**2,nbmax))
   ABI_ALLOCATE(fnltwg3,(Sigp%npwc*nspinor**2,nbmax))
 endif

 if (niter>1) then
   ABI_ALLOCATE(ddwfwfg,(nfftot_gw*nspinor**2,3,3))
   ABI_ALLOCATE(lnkp,(nfftot_gw*nspinor**2))
   ABI_ALLOCATE(kns,(Sigp%npwc*nspinor**2,Psps%mpsang*Psps%mpsang,Cryst%natom))
   ABI_ALLOCATE(fdrhotwg,(Sigp%npwc*nspinor**2,Psps%mpsang*Psps%mpsang,Cryst%natom,3))
   ABI_ALLOCATE(fkdwfg,(nfftot_gw*nspinor**2,3))
   ABI_ALLOCATE(ff,(nlx*nlx,Cryst%natom,nlx*nlx,Cryst%natom))
   ABI_ALLOCATE(vzn,(Sigp%npwc*nspinor**2,nlx*nlx,Cryst%natom))
 endif

 call wfd_get_ur(Wfs,kb,ikmq_ibz,is,wfr2)
 call calc_wfwfg(tabr_kmq,itim_kmq,nfftot_gw,ngfft_gw,wfr2,wfr2,wfwfg)

 do ibv = 1, nbmax
   call rho_tw_g(nspinor,Sigp%npwc,nfftot_gw,ndat1,ngfft_gw,1,use_padfft,igfftepsG0,gw_gbound, &
&                wfr1(:,ibv),itim_k,tabr_k,ph_mkt,spinrot_k,wfr2,itim_kmq,tabr_kmq,ph_mkmqt, &
&                spinrot_kmq,nspinor,rhotwg(:,ibv))

   if (ibv>nbhomo) then
      call calc_corr_sig_cd(Sr,Sigp%npwc,Sigp%npwx,ik_bz,ikmq_bz,ik_ibz,ikmq_ibz,ibv,kb,is,nomega, &
&                           Er%nomega,Er%nomega_r,Er%nomega_i,rhotwg(:,ibv), &
&                           Er%omega,epsm1q,npoles_missing,i_sz,vc_sqrt_qbz,sigmac)
   endif
 enddo

 if (niter>0) then

   do iat = 1, Cryst%natom
     do ilm = 1, Psps%mpsang*Psps%mpsang
       if (ilm>fnlmax(Cryst%typat(iat))) CYCLE
       if (ilm>=fnlloc(Cryst%typat(iat),1).and.ilm<=fnlloc(Cryst%typat(iat),2)) CYCLE
       call rho_tw_g(nspinor,Sigp%npwc,nfftot_gw,ndat1,ngfft_gw,1,use_padfft,igfftepsG0,gw_gbound, &
&                    fnlkr(:,ilm,iat),itim_k,tabr_k,ph_mkt,spinrot_k,wfr2,itim_kmq,tabr_kmq,ph_mkmqt, &
&                    spinrot_kmq,nspinor,fnltwg(:,ilm,iat))
     enddo
   enddo

   call calc_wfwfg(tabr_kmq,itim_kmq,nfftot_gw,ngfft_gw,wfr2,mtwkp,fnlwfg)

   call wfd_dur_isk(Wfs,Cryst,Kmesh,kb,ikmq_bz,is,npw_kmq,Gsph_max%rottbm1,dwfr)  !,ISkg,ik_ibz

   do ibv = 1, nbmax
     do i = 1, 3
       call drho_tw_g(nspinor,Sigp%npwc,nfftot_gw,ngfft_gw,1,use_padfft,igfftepsG0,gw_gbound,&
&                     wfr1(:,ibv),itim_k,tabr_k,ph_mkt,dwfr(:,i), &
&                     nspinor,drhotwg(:,ibv,i))
     enddo
     call rho_tw_g(nspinor,Sigp%npwc,nfftot_gw,ndat1,ngfft_gw,1,use_padfft,igfftepsG0,gw_gbound, &
&                  wfr1(:,ibv),itim_k,tabr_k,ph_mkt,spinrot_k,mtwkp,itim_kmq,tabr_kmq,ph_mkmqt, &
&                  spinrot_kmq,nspinor,fnltwg2(:,ibv))

     call rho_tw_g(nspinor,Sigp%npwc,nfftot_gw,ndat1,ngfft_gw,1,use_padfft,igfftepsG0,gw_gbound, &
&                  mtwk(:,ibv),itim_k,tabr_k,ph_mkt,spinrot_k,wfr2,itim_kmq,tabr_kmq,ph_mkmqt, &
&                  spinrot_kmq,nspinor,fnltwg3(:,ibv))
   enddo

   do i = 1, 3
     call calc_dwfwfg(tabr_kmq,itim_kmq,nfftot_gw,ngfft_gw,ph_mkmqt,wfr2,dwfr(:,i),dwfwfg(:,i))
     if (niter>1) then
       do j = 1, 3
         call calc_ddwfwfg(itim_kmq,nfftot_gw,ngfft_gw,dwfr(:,i),dwfr(:,j),ddwfwfg(:,i,j))
       enddo
     endif
   enddo

   if (niter>1) then

     do iat = 1, Cryst%natom
       do ilm = 1, Psps%mpsang*Psps%mpsang
         if (ilm>fnlmax(Cryst%typat(iat))) CYCLE
         if (ilm>=fnlloc(Cryst%typat(iat),1).and.ilm<=fnlloc(Cryst%typat(iat),2)) CYCLE
         do i = 1, 3
           call drho_tw_g(nspinor,Sigp%npwc,nfftot_gw,ngfft_gw,1,use_padfft,igfftepsG0,gw_gbound,&
&                         fnlkr(:,ilm,iat),itim_k,tabr_k,ph_mkt,dwfr(:,i), &
&                         nspinor,fdrhotwg(:,ilm,iat,i))
         enddo
         call rho_tw_g(nspinor,Sigp%npwc,nfftot_gw,ndat1,ngfft_gw,1,use_padfft,igfftepsG0,gw_gbound,&
&                      fnlkr(:,ilm,iat),itim_k,tabr_k,ph_mkt,spinrot_k,mtwkp,itim_kmq,tabr_kmq,ph_mkmqt, &
&                      spinrot_kmq,nspinor,kns(:,ilm,iat))
       enddo
     enddo

     do i = 1, 3
       call calc_dwfwfg(tabr_kmq,itim_kmq,nfftot_gw,ngfft_gw,ph_mkmqt,mtwkp,dwfr(:,i),fkdwfg(:,i))
     enddo
     call calc_wfwfg(tabr_kmq,itim_kmq,nfftot_gw,ngfft_gw,mtwkp,mtwkp,lnkp)

   endif

   ABI_DEALLOCATE(wfr2)
   ABI_DEALLOCATE(gwfg)
   ABI_DEALLOCATE(dwfr)
   ABI_ALLOCATE(cauxg,(Sigp%npwc,nbmax))

   cauxg(:,:)=(0.0,0.0)
   do ibv = 1, nbmax
     do ig=1,Sigp%npwc
       drhaux(:)=cmplx(real(drhotwg(ig,ibv,:)),aimag(drhotwg(ig,ibv,:)))
       cauxg(ig,ibv)=vdotw(qplg(ig,:),drhaux,Cryst%gmet,"G")
     enddo
   enddo
   do ig=1,Sigp%npwc
     cauxg(ig,:)=cauxg(ig,:)-fnltwg2(ig,:)+fnltwg3(ig,:)
   enddo

 endif

 ptwsq(:,:,:)=(0.0,0.0)
 outofbox=0
 do igp=1,Sigp%npwc
   do ig=1,Sigp%npwc
     gmgp(:)=Gsph_c%gvec(:,igp)-Gsph_c%gvec(:,ig)
     if (ANY(gmgp(:)>ngfft_gw(1:3)/2) .or. ANY(gmgp(:)<-(ngfft_gw(1:3)-1)/2)) then
       outofbox = outofbox+1; CYCLE
     end if
     ig4x= modulo(gmgp(1),ngfft_gw(1))
     ig4y= modulo(gmgp(2),ngfft_gw(2))
     ig4z= modulo(gmgp(3),ngfft_gw(3))
     ig4= 1+ig4x+ig4y*ngfft_gw(1)+ig4z*ngfft_gw(1)*ngfft_gw(2)

     ig5x= modulo(-gmgp(1),ngfft_gw(1))
     ig5y= modulo(-gmgp(2),ngfft_gw(2))
     ig5z= modulo(-gmgp(3),ngfft_gw(3))
     ig5= 1+ig5x+ig5y*ngfft_gw(1)+ig5z*ngfft_gw(1)*ngfft_gw(2)

     if (igp>=ig) then
       ptwsq(ig,igp,1)=wfwfg(ig4)
     endif
     if (niter>0) then
       drhaux(:)=cmplx(real(dwfwfg(ig4,:)),aimag(dwfwfg(ig4,:)))
       ptwsq(ig,igp,2)=ptwsq(ig,igp,2) + vdotw(qplg(igp,:),drhaux,Cryst%gmet,"G")-fnlwfg(ig4)
       if (niter>1.and.igp>=ig) then
         paux2(:)=czero
         do i = 1, 3
           drhaux(:)=cmplx(real(ddwfwfg(ig4,:,i)),aimag(ddwfwfg(ig4,:,i)))
           paux2(i)=paux2(i) + vdotw(qplg(ig,:),drhaux,Cryst%gmet,"G")
         enddo
         drhaux(:)=paux2(:)-cmplx(real(fkdwfg(ig4,:)),aimag(fkdwfg(ig4,:)))
         ptwsq(ig,igp,3)=ptwsq(ig,igp,3) + vdotw(qplg(igp,:),drhaux,Cryst%gmet,"G")
         drhaux(:)=cmplx(real(fkdwfg(ig5,:)),-aimag(fkdwfg(ig5,:)))
         ptwsq(ig,igp,3)=ptwsq(ig,igp,3) - vdotw(qplg(ig,:),drhaux,Cryst%gmet,"G")+lnkp(ig4)
       endif
     endif
   enddo
 enddo

 if (outofbox/=0) then
   enough=enough+1
   if (enough<=10) then
     write(msg,'(a,i5)')' Number of G1-G2 pairs outside the G-sphere for Wfns = ',outofbox
     MSG_WARNING(msg)
     if (enough==10) then
       write(msg,'(a)')' ========== Stop writing Warnings =========='
       call wrtout(std_out,msg,'COLL')
     end if
   end if
 end if

 do ibv = 1, nbmax
   do igp=1,Sigp%npwc
     do ig=1,igp
       ptwsq(ig,igp,1)=ptwsq(ig,igp,1)-conjg(rhotwg(ig,ibv))*rhotwg(igp,ibv)
     enddo
   end do !igp
 end do !ig

 do igp = 1, Sigp%npwc
   do ig = igp+1, Sigp%npwc
     ptwsq(ig,igp,1)=conjg(ptwsq(igp,ig,1))
   enddo
 enddo

 if (niter>0) then
   ABI_ALLOCATE(paux,(Sigp%npwc,Sigp%npwc))
   paux(:,:)=(0.0,0.0)
   minusone=(-1.,0.)
   do ibv = 1, nbmax
     call XGERC(Sigp%npwc,Sigp%npwc,minusone,conjg(rhotwg(:,ibv)),1,conjg(cauxg(:,ibv)),1,ptwsq(:,:,2),Sigp%npwc)
   enddo
   do igp=1,Sigp%npwc
     do ig=1,Sigp%npwc
       if (ig>=igp) then
         do iat = 1, Cryst%natom
           do ilm = 1, nlx*nlx
             if (ilm>fnlmax(Cryst%typat(iat))) CYCLE
             if (ilm>=fnlloc(Cryst%typat(iat),1).and.ilm<=fnlloc(Cryst%typat(iat),2)) CYCLE
             paux(ig,igp)=paux(ig,igp)+conjg(fnltwg(ig,ilm,iat))*fnltwg(igp,ilm,iat)
           enddo
         enddo
       endif
       ptwsq(ig,igp,2)=ptwsq(ig,igp,2)+ptwsq(ig,igp,1)*kplqg(igp)
     end do !igp
   end do !ig
   do ig=1,Sigp%npwc
     do igp=1,Sigp%npwc
       if (ig>=igp) then
         ptwsq(ig,igp,2)=ptwsq(ig,igp,2)+paux(ig,igp)
       else
         ptwsq(ig,igp,2)=ptwsq(ig,igp,2)+conjg(paux(igp,ig))
       endif
     end do !igp
   end do !ig
   ABI_DEALLOCATE(paux)
 endif

 if (niter>1) then

   call compute_ff(Cryst,Kmesh,Psps,ikmq_bz,nlx,istwf_kmq,npw_kmq,kg_kmq,Gsph_max%rottbm1,fnlmax,fnlloc,ff)

   vzn(:,:,:)=(0.0,0.0)
   do igp=1,Sigp%npwc
     do iat = 1, Cryst%natom
       do ilm = 1, nlx*nlx
         if (ilm>fnlmax(Cryst%typat(iat))) CYCLE
         if (ilm>=fnlloc(Cryst%typat(iat),1).and.ilm<=fnlloc(Cryst%typat(iat),2)) CYCLE
         vzn(igp,:,:) = vzn(igp,:,:) + half*ff(ilm,iat,:,:)*fnltwg(igp,ilm,iat)
       enddo
     enddo
     do iat = 1, Cryst%natom
       do ilm = 1, nlx*nlx
         if (ilm>fnlmax(Cryst%typat(iat))) CYCLE
         if (ilm>=fnlloc(Cryst%typat(iat),1).and.ilm<=fnlloc(Cryst%typat(iat),2)) CYCLE
         drhaux(:)=cmplx(real(fdrhotwg(igp,ilm,iat,:)),aimag(fdrhotwg(igp,ilm,iat,:)))
         vzn(igp,ilm,iat)=vzn(igp,ilm,iat)+vdotw(qplg(igp,:),drhaux,Cryst%gmet,"G")-kns(igp,ilm,iat)
       enddo
     enddo
     do ig=1,igp
       do ibv = 1, nbmax
         ptwsq(ig,igp,3)=ptwsq(ig,igp,3)-conjg(cauxg(ig,ibv))*cauxg(igp,ibv)
       enddo
       ptwsq(ig,igp,3)=ptwsq(ig,igp,3)+kplqg(igp)*ptwsq(ig,igp,2)+kplqg(ig)*conjg(ptwsq(igp,ig,2))&
&                                     -kplqg(ig)*ptwsq(ig,igp,1)*kplqg(igp)
       do iat = 1, Cryst%natom
         do ilm = 1, nlx*nlx
           if (ilm>fnlmax(Cryst%typat(iat))) CYCLE
           if (ilm>=fnlloc(Cryst%typat(iat),1).and.ilm<=fnlloc(Cryst%typat(iat),2)) CYCLE
           ptwsq(ig,igp,3)=ptwsq(ig,igp,3)+conjg(fnltwg(ig,ilm,iat))*vzn(igp,ilm,iat)+ &
&                                                fnltwg(igp,ilm,iat)*conjg(vzn(ig,ilm,iat))
         enddo
       enddo
     end do !igp
   end do !ig

   do igp = 1, Sigp%npwc
     do ig = igp+1, Sigp%npwc
       ptwsq(ig,igp,3)=conjg(ptwsq(igp,ig,3))
     enddo
   enddo

 endif

 ABI_DEALLOCATE(rhotwg)
 ABI_DEALLOCATE(wfwfg)
 if (niter>0) then
   ABI_DEALLOCATE(cauxg)
   ABI_DEALLOCATE(drhotwg)
   ABI_DEALLOCATE(dwfwfg)
   ABI_DEALLOCATE(fnltwg)
   ABI_DEALLOCATE(fnltwg2)
   ABI_DEALLOCATE(fnltwg3)
   ABI_DEALLOCATE(fnlwfg)
 endif
 if (niter>1) then
   ABI_DEALLOCATE(ddwfwfg)
   ABI_DEALLOCATE(lnkp)
   ABI_DEALLOCATE(kns)
   ABI_DEALLOCATE(fdrhotwg)
   ABI_DEALLOCATE(fkdwfg)
   ABI_DEALLOCATE(vzn)
   ABI_DEALLOCATE(ff)
 endif

end subroutine fft4eet_sig_kb_cd
!!***

!----------------------------------------------------------------------

!!****f* ABINIT/calc_corr_sig_cd
!! NAME
!! calc_corr_sig_cd
!!
!! FUNCTION
!! Calculate contributions to the self-energy operator with the contour deformation method using the EET.
!!
!! COPYRIGHT
!! Copyright (C) 1999-2013 ABINIT group (FB, GMR, VO, LR, RWG, MG)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!  nomega=Total number of frequencies where $\Sigma_c$ matrix elements are evaluated.
!!  nomegae=Number of frequencies where $\epsilon^{-1}$ has been evaluated.
!!  nomegaei=Number of imaginary frequencies for $\epsilon^{-1}$ (non zero).
!!  nomegaer=Number of real frequencies for $\epsilon^{-1}$
!!  npwc=Number of G vectors for the correlation part.
!!  npwx=Number of G vectors in rhotwgp for each spinorial component.
!!  nspinor=Number of spinorial components.
!!  theta_mu_minus_e0i=1 if e0i is occupied, 0 otherwise. Fractional occupancy in case of metals. 
!!  omegame0i(nomega)=Contains $\omega-\epsilon_{k-q,b1,\sigma}$
!!  epsm1q(npwc,npwc,nomegae)=Symmetrized inverse dielectric matrix (exchange part is subtracted).
!!  omega(nomegae)=Set of frequencies for $\epsilon^{-1}$.
!!  rhotwgp(npwx*nspinor)=Matrix elements: $<k-q,b1,\sigma|e^{-i(q+G)r} |k,b2,\sigma>*vc_sqrt$
!!
!! OUTPUT
!! ket(npwc,nomega)=Contains \Sigma_c(\omega)|\phi> in reciprocal space. 
!!
!! SIDE EFFECTS
!! npoles_missing=Incremented with the number of poles whose contribution has not been taken into account due to
!!  limited frequency mesh used for W.
!!
!! PARENTS
!!      calc_sig_cd_eet
!!
!! CHILDREN
!!      spline,splint,xgemm,xgemv
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

subroutine calc_corr_sig_cd(Sr,npwc,npwx,ik_bz,ikmq_bz,ik_ibz,ikmq_ibz,ibv,kb,is,nomega,nomegae, &
&                           nomegaer,nomegaei,rhotwg,omega,epsm1q,npoles_missing,i_sz,vc_sqrt_qbz,sigmac)

 use defs_basis
 use m_profiling
 use m_timer
 use m_splines
 use m_xomp

 use m_gwdefs, only : czero_gw, cone_gw
 use m_blas,   only : xgemv, xgemm, xdotc
 use m_sigma_results,   only : sigma_results

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'calc_corr_sig_cd'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 type(Sigma_results),intent(in) :: Sr

 integer,intent(in) :: nomega,nomegae,nomegaei,nomegaer,npwc,npwx
 integer,intent(in) :: ik_bz,ikmq_bz,ik_ibz,ikmq_ibz,ibv,kb,is
 integer,intent(inout) :: npoles_missing
!arrays
 real(dp),intent(in) :: i_sz
 complex(dpc),intent(in) :: omega(nomegae)
 complex(gwpc) :: epsm1q(npwc,npwc,nomegae) 
 complex(gwpc),intent(in) :: rhotwg(npwx)
 complex(gwpc),intent(in) :: vc_sqrt_qbz(npwc)
 complex(dpc),intent(inout) :: sigmac(nomega)

!Local variables-------------------------------
!scalars
 integer :: ig,io,ios,my_err,ierr
 real(dp) :: rt_imag,rt_real,local_one,local_zero
 complex(dpc) :: ct,domegaleft,domegaright
 complex(gwpc) :: fact
!arrays
 real(dp) :: omegame0i(nomega),omegame0i_tmp(nomega)
 real(dp) :: tmp_x(2),tmp_y(2)
 real(dp) :: left(nomega),right(nomega)
 real(dp) :: rtmp_r(nomegaer),rtmp_i(nomegaer)
 complex(dpc) :: omega_imag(nomegaei+1)
 complex(gwpc) :: epsrho(npwc,nomegae),epsrho_imag(npwc,nomegaei+1)
 complex(gwpc) :: weight(nomegaei+1,nomega)
 logical :: my_calc_poles(nomega)
!
 complex(gwpc),allocatable :: rtaux(:)
 complex(gwpc),allocatable :: ket(:,:)
!*************************************************************************

 DEV_TIMER_START("")

 ABI_ALLOCATE(rtaux,(npwc))
 ABI_ALLOCATE(ket,(npwc,nomega))

 my_calc_poles=.TRUE.
 my_err=0

 ! Avoid divergences in $\omega - \omega_s$.
 omegame0i(:) = real(Sr%omega4sd(kb,ikmq_ibz,:,is)) - Sr%e0(ibv,ik_ibz,is)
 omegame0i_tmp(:)=omegame0i(:)
 do ios=1,nomega
   if (ABS(omegame0i_tmp(ios))<tol6) omegame0i_tmp(ios)=sign(tol6,omegame0i_tmp(ios))
 end do

 do ig = 1,npwc
   rtaux(ig)=rhotwg(ig)*vc_sqrt_qbz(ig)
 enddo
 if (ik_bz==ikmq_bz) then
   rtaux(1)=czero_gw
   if (ibv==kb) then
     rtaux(1)=cmplx(sqrt(i_sz),0.0_gwp)
   endif
 endif

 !
 ! Calculate $ \sum_{Gp} (\epsilon^{-1}_{G Gp}(\omega)-\delta_{G Gp}) \rhotwgp(Gp) $
!$omp parallel do
 do io=1,nomegae
   call XGEMV('N',npwc,npwc,cone_gw,epsm1q(:,:,io),npwc,rtaux(:),1,czero_gw,epsrho(:,io),1)
 end do

 ! Integrand along the imaginary axis.
 epsrho_imag(:,1)=epsrho(:,1)
 epsrho_imag(:,2:nomegaei+1)=epsrho(:,nomegaer+1:nomegae)

 ! Frequency mesh for integral along the imaginary axis.    
 omega_imag(1)=omega(1)
 omega_imag(2:nomegaei+1)=omega(nomegaer+1:nomegae)

 weight(1,:) = ATAN(-half*AIMAG(omega_imag(2))/REAL(omegame0i_tmp(:)))
 domegaleft  = (three*omega_imag(nomegaei+1)-omega_imag(nomegaei))
 domegaright = (omega_imag(nomegaei+1)+omega_imag(nomegaei))
 right(:)    = -AIMAG(omega_imag(nomegaei+1)-omega_imag(nomegaei))*REAL(omegame0i_tmp(:))
 left(:)     = quarter*AIMAG(domegaleft)*AIMAG(domegaright) &
                +REAL(omegame0i_tmp(:))*REAL(omegame0i_tmp(:))
 weight(nomegaei+1,:) = ATAN(right(:)/left(:))
 ! Calculate the rest of the weights
 do io=2,nomegaei
   domegaleft  = (omega_imag(io  )+omega_imag(io-1))
   domegaright = (omega_imag(io+1)+omega_imag(io  ))
   right(:)    = -half*AIMAG(omega_imag(io+1)-omega_imag(io-1))*REAL(omegame0i_tmp(:))
   left(:)     = REAL(omegame0i_tmp(:))*REAL(omegame0i_tmp(:)) &
    +quarter*AIMAG(domegaleft)*AIMAG(domegaright)
   weight(io,:) = ATAN(right(:)/left(:))
 end do

 ! Use BLAS call to perform matrix-matrix multiplication and accumulation
 fact = CMPLX(piinv,zero) 

 ket = 0.0_gwp

 call xgemm('N','N',npwc,nomega,nomegaei+1,fact,epsrho_imag,npwc,&
  weight,nomegaei+1,cone_gw,ket(1:npwc,:),npwc)

 local_one = one
 local_zero = zero

 ! ============================================
 ! ==== Add contribution coming from poles ====
 ! ============================================
 ! First see if the contribution has been checked before the routine is entered
 do ios=1,nomega
   if (omegame0i_tmp(ios)>tol12) then
     if (local_one<tol12) my_calc_poles(ios) = .FALSE.
   end if
   if (omegame0i_tmp(ios)<-tol12) my_calc_poles(ios) = .FALSE.
 end do !ios

 if (ANY(my_calc_poles(:))) then ! Make sure we only enter if necessary

! *** OPENMP SECTION *** Added by MS
!!OMP !write(std_out,'(a,i0)') ' Entering openmp loop. Number of threads: ',xomp_get_num_threads()
!$OMP PARALLEL SHARED(npwc,nomega,nomegaer,local_one,local_zero, &
!$OMP                    omega,epsrho,omegame0i_tmp,ket,my_calc_poles) &
!$OMP PRIVATE(ig,ios,rtmp_r,rtmp_i,tmp_x,tmp_y,rt_real,rt_imag,ct,ierr) REDUCTION(+:my_err) 
!!OMP $ write(std_out,'(a,i0)') ' Entering openmp loop. Number of threads: ',xomp_get_num_threads()
!$OMP DO 
   do ig=1,npwc
     !
     ! * Prepare the spline interpolation by filling at once the arrays rtmp_r, rtmp_i
     call spline(DBLE(omega(1:nomegaer)),DBLE(epsrho(ig,1:nomegaer)),nomegaer,local_zero,local_zero,rtmp_r)
     call spline(DBLE(omega(1:nomegaer)),DBLE(AIMAG(epsrho(ig,1:nomegaer))),nomegaer,local_zero,local_zero,rtmp_i)

     !! call spline_complex( DBLE(omega(1:nomegaer)), epsrho(ig,1:nomegaer), nomegaer, zero, zero, rtmp )

     do ios=1,nomega

       if (.NOT.my_calc_poles(ios)) CYCLE

       ! * Interpolate real and imaginary part of epsrho at |omegame0i_tmp|.
       tmp_x(1) = ABS(omegame0i_tmp(ios))
       call splint(nomegaer,DBLE(omega(1:nomegaer)),DBLE(epsrho(ig,1:nomegaer)),rtmp_r,1,tmp_x,tmp_y,ierr=ierr)
       if (ig==1) my_err = my_err + ierr
       rt_real = tmp_y(1)

       tmp_x(1) = ABS(omegame0i_tmp(ios))
       call splint(nomegaer,DBLE(omega(1:nomegaer)),DBLE(AIMAG(epsrho(ig,1:nomegaer))),rtmp_i,1,tmp_x,tmp_y)
       rt_imag = tmp_y(1)

       !!call splint_complex(nomegaer,DBLE(omega(1:nomegaer)),epsrho(ig,1:nomegaer),rtmp,1,tmp_x,yfit)

       ct=DCMPLX(rt_real,rt_imag)

       ket(ig,ios)=ket(ig,ios)+ct*local_one

     end do !ios
   end do !ig
!$OMP END DO
!$OMP END PARALLEL
 end if ! ANY(my_calc_poles)

 do ios=1,nomega
   sigmac(ios)=sigmac(ios) + XDOTC(npwc,rtaux(:),1,ket(:,ios),1)
 end do

 npoles_missing = npoles_missing + my_err

 ABI_DEALLOCATE(rtaux)
 ABI_DEALLOCATE(ket)

 DEV_TIMER_STOP("")

end subroutine calc_corr_sig_cd
!!***
